﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Threading.Tasks;

namespace LampdaExpressions
{
    class Program
    {
        static void Main(string[] args)
        {
            Car car1 = new Car(2014, "BMW");
            Car car2 = new Car(2015, "Audi");
            Car car3 = new Car(2015, "Mercedes");

            List<Car> cars = new List<Car>();
            cars.Add(car1);
            cars.Add(car2);
            cars.Add(car3);

            // Method 1 - Traditional way
            Predicate<Car> carPredicate = new Predicate<Car>(CarModelSearch);

            _searchCriteria = "Audi";
            var car = cars.Find(carPredicate);
            Console.WriteLine(car);


            // Method 2 - Delegate / Anonymous function
            _searchCriteria = "BMW";
            car = cars.Find(delegate(Car c) { return c.Model.Equals(_searchCriteria); });
            Console.WriteLine(car);


            // Method 3 - Lampda Expression
            _searchCriteria = "Mercedes";
            car = cars.Find(c => c.Model.Equals(_searchCriteria));
            Console.WriteLine(car);


            // Search - using FindAll() and Lamda Expressions
            var searchYear = 2014;
            var result = cars.FindAll(c => c.Year > searchYear);
            foreach (var c in result)
            {
                Console.WriteLine(c);
            }

            // Search - using FindIndex(startIndex, count, Predicate) and Lamda Expressions to 
            // find the index of the first occurrence (search-area from startIndex and count number of elements forwards)
            searchYear = 2015;
            int findIndex = cars.FindIndex(0, 2, c => c.Year == searchYear);
            Console.WriteLine("First index of " + searchYear + ": " + findIndex);

            
        }

        private static string _searchCriteria;
        static bool CarModelSearch(Car car)
        {
            return car.Model.Equals(_searchCriteria);
        }

        public class Car
        {
            public int Year { get; set; }
            public string Model { get; set; }

            public Car(int year, string model)
            {
                Year = year;
                Model = model;
            }

            public override string ToString()
            {
                return string.Format("Year: {0}, Model: {1}", Year, Model);
            }
        }
    }
}
